@@ -1,3 +1,17 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::Test;
 
 use strict;
@@ -8,7 +22,7 @@ use Exporter ();
 use Config;
 use Apache::TestConfig ();
 
-use vars qw(@ISA @EXPORT $VERSION %SubTests @SkipReasons);
+use vars qw(@ISA @EXPORT %EXPORT_TAGS $VERSION %SubTests @SkipReasons);
 
 @ISA = qw(Exporter);
 @EXPORT = qw(ok skip sok plan have have_lwp have_http11
@@ -16,7 +30,13 @@ use vars qw(@ISA @EXPORT $VERSION %SubTests @SkipReasons);
              have_min_apache_version have_apache_version have_perl 
              have_min_perl_version have_min_module_version
              have_threads under_construction have_apache_mpm);
-$VERSION = '1.08';
+
+# everything but ok(), skip(), and plan() - Test::More provides these
+my @test_more_exports = grep { ! /^(ok|skip|plan)$/ } @EXPORT;
+
+%EXPORT_TAGS = (withtestmore => \@test_more_exports);
+
+$VERSION = '1.11';
 
 %SubTests = ();
 @SkipReasons = ();
@@ -32,7 +52,7 @@ sub config {
 }
 
 sub vars {
-    config()->{vars};
+    @_ ? @{ config()->{vars} }{ @_ } : config()->{vars};
 }
 
 sub sok (&;$) {
@@ -225,8 +245,7 @@ sub have_min_module_version {
     # have_module requires the perl module
     return 0 unless have_module($module);
 
-    my $has_version = $module->VERSION || 0;
-    return 1 if $has_version >= $version;
+    return 1 if eval { $module->VERSION($version) };
 
     push @SkipReasons, "$module version $version or higher is required";
     return 0;
@@ -264,7 +283,7 @@ sub have_min_apache_version {
     my $cfg = Apache::Test::config();
     (my $current) = $cfg->{server}->{version} =~ m:^Apache/(\d\.\d+\.\d+):;
 
-    if ($current lt $wanted) {
+    if (normalize_vstring($current) < normalize_vstring($wanted)) {
         push @SkipReasons,
           "apache version $wanted or higher is required," .
           " this is version $current";
@@ -280,7 +299,7 @@ sub have_apache_version {
     my $cfg = Apache::Test::config();
     (my $current) = $cfg->{server}->{version} =~ m:^Apache/(\d\.\d+\.\d+):;
 
-    if ($current ne $wanted) {
+    if (normalize_vstring($current) != normalize_vstring($wanted)) {
         push @SkipReasons,
           "apache version $wanted or higher is required," .
           " this is version $current";
@@ -372,6 +391,16 @@ sub under_construction {
     return 0;
 }
 
+# normalize Apache-sytle version strings (2.0.48, 0.9.4)
+# for easy numeric comparison.  note that 2.1 and 2.1.0
+# are considered equivalent.
+sub normalize_vstring {
+
+    my @digits = shift =~ m/(\d+)\.?(\d*)\.?(\d*)/;
+
+    return join '', map { sprintf("%03d", $_ || 0) } @digits;
+}
+
 package Apache::TestToString;
 
 sub TIEHANDLE {
@@ -682,8 +711,53 @@ reference, but be careful that the keys will be different.
 
 Also see plan().
 
+=item config
+
+  my $cfg = Apache::Test::config();
+  my $server_rev = $cfg->{server}->{rev};
+  ...
+
+C<config()> gives an access to the configuration object.
+
+=item vars
+
+  my $serverroot = Apache::Test::vars->{serverroot};
+  my $serverroot = Apache::Test::vars('serverroot');
+  my($top_dir, $t_dir) = Apache::Test::vars(qw(top_dir t_dir));
+
+C<vars()> gives an access to the configuration variables, otherwise
+accessible as:
+
+  $vars = Apache::Test::config()->{vars};
+
+If no arguments are passed, the reference to the variables hash is
+returned. If one or more arguments are passed the corresponding values
+are returned.
+
 =back
 
+=head1 Test::More Integration
+
+There are a few caveats if you want to use I<Apache::Test> with 
+I<Test::More> instead of the default I<Test> backend.  The first is
+that I<Test::More> requires you to use its own C<plan()> function
+and not the one that ships with I<Apache::Test>.  I<Test::More> also
+defines C<ok()> and C<skip()> functions that are different, and 
+simply C<use>ing both modules in your test script will lead to redefined
+warnings for these subroutines.
+
+To assist I<Test::More> users we have created a special I<Apache::Test>
+import tag, C<:withtestmore>, which will export all of the standard
+I<Apache::Test> symbols into your namespace except the ones that collide
+with I<Test::More>.
+
+    use Apache::Test qw(:withtestmore);
+    use Test::More;
+
+    plan tests => 1;           # Test::More::plan()
+
+    ok ('yes', 'testing ok');  # Test::More::ok()
+
 =head1 Apache::TestToString Class
 
 The I<Apache::TestToString> class is used to capture I<Test.pm> output
